#include "ioutils.h"

#include <stdarg.h>
#include <string.h>
#include <stdio.h>

/*
 * If these flags are not 0, output to stdout, resp. stderr
 * is logged into a file.
 */
int32 g_stdoutLog = 1;
int32 g_stderrLog = 1;
char g_stdoutLogFileName[256]; /* !!! */
char g_stderrLogFileName[256]; /* !!! */

/*
 * If this flag is 0, any output to a terminal is ignored.
 * Slaves should set this to 0.
 */
int32 g_stdoutTermOutput = 1;
int32 g_stderrTermOutput = 1;

/*
 * This should be set to the thread number.
 */
int32 g_threadRankForOutput;

/*
 * Error output.
 */
void errput( const char *what, ... )
{
  va_list ap;
  char buf[256]; /* !!! */
  FILE *file;

  if ((!g_stderrTermOutput) && (!g_stderrLog)) {
    return;
  }
  va_start( ap, what );
  sprintf( buf, "\nError: Thread %d: %s\n\n", g_threadRankForOutput, what );
  if (g_stderrTermOutput) {
    vfprintf( stdout, buf, ap );
  }
  if (g_stderrLog) {
    file = fopen( g_stderrLogFileName, "a" );
    vfprintf( file, buf, ap );
    fclose( file );
  }
  va_end( ap );
}

/*
 * Normal output.
 */
void output( const char *what, ... )
{
  va_list ap;
  char buf[256]; /* !!! */
  FILE *file;

  if ((!g_stdoutTermOutput) && (!g_stdoutLog)) {
    return;
  }
  va_start( ap, what );
  sprintf( buf, "Thread %d: %s\n", g_threadRankForOutput, what );
  if (g_stdoutTermOutput) {
    vfprintf( stdout, buf, ap );
  }
  if (g_stdoutLog) {
    file = fopen( g_stdoutLogFileName, "a" );
    vfprintf( file, buf, ap );
    fclose( file );
  }
  va_end( ap );
}
