#include "ops_d_la.h"
#include "ops_la.h"
#include "ioutils.h"

/*
 * Distributed linear algebra operations.
 */

/*
 * Refresh ditributed array according to communication lists.
 */
int32 d_la_refreshFloatArray( float64 *outer, float64 *inner,
			      CommList *clI, CommList *clO, float64 *buf )
{
  int32 i, j;
  int32 size;
  int32 ret = 0;

#ifdef DEBUG_D_LA
  output( "Refresh float array." );
#endif

  /*
   * Send inner values to out list members.
   */
  for (i = 0; i < clO->nList; i++) {
    for (j = 0; j < clO->cs[i].size; j++) {
      buf[j] = inner[clO->cs[i].set[j]];
#ifdef DEBUG_D_LA
      output( "out: %f", buf[j] );
#endif
    }
    ret = itc_sendArray( buf, clO->cs[i].size, ITC_TypeFloat64,
			 clO->cs[i].rank, ITC_TagArray );
    if (ret) goto dla_rfa_end;
  }

  /*
   * Receive outer values to in list members.
   */
  for (i = 0; i < clI->nList; i++) {
    ret = itc_recvArrayNA( buf, &size, clI->cs[i].size, ITC_TypeFloat64,
			   clI->cs[i].rank, ITC_TagArray );
    if (ret) goto dla_rfa_end;
    for (j = 0; j < clI->cs[i].size; j++) {
      outer[clI->cs[i].set[j]] = buf[j];
#ifdef DEBUG_D_LA
      output( "in: %f", buf[j] );
#endif
    }
  }

 dla_rfa_end:
  return( ret );
}

/*
 * Global dot product.
 */
int32 d_la_dot_v_v( float64 *dot, Vector *v1, Vector *v2 )
{
  int32 ret = 0;
  float64 localDot;

  la_dot_v_v( &localDot, v1, v2 );
  
  itc_reduce( &localDot, dot, 1, ITC_TypeFloat64,
	      ITC_ReduceSum, ITC_ReduceRootAll );

  return( ret );
}
