#ifndef _OPS_LA_H_
#define _OPS_LA_H_

#include "matrix_csr.h"
#include "matrix.h"
#include "vector.h"

/*
 * Test sizes for matrix - vector multiplying.
 */
int32 la_testSizes_mul_m_v( int32 nRow, int32 nCol, int32 n1, int32 n2 );
/*
 * Test sizes for vector - vector sum.
 */
int32 la_testSizes_add_v_v( int32 n1, int32 n2, int32 n3 );
/*
 * v_1 = M v_2. Sizes are checked.
 */
int32 la_mul_m_v( Vector *v1, Matrix *m, Vector *v2 );
/*
 * v_1 = M^T v_2. Sizes are checked.
 */
int32 la_mul_mT_v( Vector *v1, Matrix *m, Vector *v2 );
/*
 * v_1 = M v_2. M has CSR storage (no check).
 */
int32 la_mul_CSR_m_v( Vector *v1, Matrix *m, Vector *v2 );
/*
 * v_1 = M^T v_2. M has CSR storage (no check).
 */
int32 la_mul_CSR_mT_v( Vector *v1, Matrix *m, Vector *v2 );
/*
 * v_1 = v_2 + v_3. Sizes are checked.
 */
int32 la_add_v_v( Vector *v1, Vector *v2, Vector *v3 );
/*
 * v_1 = v_2 - v_3. Sizes are checked.
 */
int32 la_sub_v_v( Vector *v1, Vector *v2, Vector *v3 );
/*
 * v_1 = a v_2 + v_3. Sizes are checked.
 */
int32 la_mul_add_f_v_v( Vector *v1, float64 a, Vector *v2, Vector *v3 );
/*
 * dot = (v_1, v_2). Sizes are checked.
 */
int32 la_dot_v_v( float64 *dot, Vector *v1, Vector *v2 );
/*
 * dot = (mask v_1, v_2). Sizes are checked.
 */
int32 la_mask_dot_v_v( float64 *dot, int32 *mask, Vector *v1, Vector *v2 );
/*
 * Fill vector with a constant.
 */
int32 la_fill_v( Vector *v, float64 val );
/*
 * v_1 = v_2. Sizes are checked.
 */
int32 la_copy_v( Vector *v1, Vector *v2 );

#endif
