#include "solver.h"
#include "ops_la.h"
#include "ioutils.h"

#include <math.h>
#include <stdio.h>

/*
 * Solve A x = b by preconditioned CG. No checks.
 */
int32 sol_PCG( Vector *x, Matrix *a, Vector *b, void *precond,
	       float64 eps, uchar check )
{
  int32 ret;
  Vector *r, *d, *p, *s;
  float64 rho, rho0, alpha, beta, err, err0, errsr = 0.0;
  int32 i, iMax;

  output( "PCG." );
 
  if ((ret = v_createAlloc( &r, x->nItem ))) return( ret );
  if ((ret = v_createAlloc( &d, x->nItem ))) return( ret );
  if ((ret = v_createAlloc( &p, x->nItem ))) return( ret );
  if ((ret = v_createAlloc( &s, x->nItem ))) return( ret );
  
  la_fill_v( x, 0.0 );

  la_copy_v( r, b );

  /* Initial error. */
  la_dot_v_v( &err0, r, r );
  err0 = sqrt( err0 );
  output( "Initial error: %.12f", err0 );

  /* Preonditioning */
  la_copy_v( s, r );

  la_copy_v( d, s );

  la_dot_v_v( &rho0, r, s );
  rho = rho0;
#ifdef DEBUG_SOL
  output( "rho0: %f", rho0 );
#endif

  i = 0;
  iMax = 300;

  while (i < iMax) {
    
    la_mul_m_v( p, a, d );
    la_dot_v_v( &alpha, p, d );
    if (alpha == 0) {
      break;
    }
    alpha = rho / alpha;

#ifdef DEBUG_SOL
    output( "alpha: %f", alpha );
#endif
    
    la_mul_add_f_v_v( x, alpha, d, x );
    la_mul_add_f_v_v( r, -alpha, p, r );
    
    i++;
    la_dot_v_v( &err, r, r );
    errsr = sqrt( err );
    
#ifdef DEBUG_SOL
    output( "%d %.12f", i, err );
#endif
    if ((errsr / err0) < eps) break;
    
    /* Preonditioning */
    la_copy_v( s, r );
    
    la_dot_v_v( &rho, r, s );
    beta = rho / rho0;
    rho0 = rho;
    
    la_mul_add_f_v_v( d, beta, d, s );

  }

  output( "Checking residual." );
  la_copy_v( r, b );
  la_mul_m_v( s, a, x );
  la_sub_v_v( r, r, s );
  la_dot_v_v( &err, r, r );
  output( "Checking residual done." );

  v_freeDestroy( &r );
  v_freeDestroy( &d );
  v_freeDestroy( &p );
  v_freeDestroy( &s );
  
  if ((i == iMax) && ((errsr / err0) > eps)) {
    output( "we did not converge!" );
    output( "-> %d iterations with error %.12f is too much!",
	    i, errsr );
    return( 0 );
  }
  output( "convergence at %d, %.12f", i, errsr );
  output( "true error: %.12f", sqrt( err ) );
  
  return( 0 );
}
