#include "timer.h"

/*
 * Timer manipulation routines.
 */

void timerReset( Timer *t )
{
  t->ticksStart = t->ticksEnd = 0;
  t->userStart = t->userEnd = 0;
  t->systemStart = t->systemEnd = 0;
}   

void timerActivate( Timer *t )
{
  struct tms tm;

  t->ticksStart = times( &tm );
  t->userStart = tm.tms_utime;
  t->systemStart = tm.tms_stime;
}

void timerStart( Timer *t )
{
  timerReset( t );
  timerActivate( t );
}

void timerPause( Timer *t )
{
  struct tms tm;

  t->ticksEnd += times( &tm ) - t->ticksStart;
  t->userEnd += tm.tms_utime - t->userStart;
  t->systemEnd += tm.tms_stime - t->systemStart;
}

void timerCurrentStatus( Timer *t, clock_t *ticks,
			 time_t *user, time_t *system )
{
  struct tms tm;

  *ticks = times( &tm ) - t->ticksStart;
  *user = tm.tms_utime - t->userStart;
  *system = tm.tms_stime - t->systemStart;
}
