#ifndef _VECTOR_H_
#define _VECTOR_H_

#include "types.h"

typedef struct {
  int32 nItem;
  int32 nAlloc;
  float64 *val;
} Vector;

/*
 * Allocate vector structure.
 */
Vector *allocVector();
/*
 * Free vector structure.
 */
void freeVector( Vector *v );
/*
 * Create empty vector.
 */
int32 v_create( Vector **pv );
/*
 * Allocate space for 'n' items in vector 'v' (must be allocated).
 * Sets no. of items to 'n'.
 */
int32 v_alloc( Vector *v, int32 n );
/*
 * Create and allocate vector.
 */
int32 v_createAlloc( Vector **pv, int32 n );
/*
 * Free vector data.
 */
int32 v_free( Vector *v );
/*
 * Free vector structure.
 */
int32 v_destroy( Vector **pv );
/*
 * Free and destroy vector.
 */
int32 v_freeDestroy( Vector **pv );
/*
 * Save vector to a file.
 */
int32 v_save( Vector *v, const char *fileName );
/*
 * Load vector from a file. Vector is created and allocated here.
 * Value indices must be correct (no check of duplicity).
 */
int32 v_load( Vector **pv, const char *fileName );

#endif
